"""Init for Brunt."""
import sys
from importlib.metadata import PackageNotFoundError, version  # pragma: no cover

try:
    __version__ = version(__name__)
except PackageNotFoundError:  # pragma: no cover
    __version__ = "unknown"
finally:
    del version, PackageNotFoundError

from .client import (  # pylint: disable=wrong-import-position
    BruntClient,
    BruntClientAsync,
)
from .thing import Thing  # pylint: disable=wrong-import-position



"""Class for thing objects."""
from __future__ import annotations

import logging
from dataclasses import dataclass, fields
from datetime import datetime
from typing import Any

# state: {'TIMESTAMP': '1619179952875', 'NAME': 'Blind', 'SERIAL': '00140d6f1950f166', 'MODEL': 'BEAKR1601', 'requestPosition': '100', 'currentPosition': '100', 'moveState': '0', 'setLoad': '2418', 'currentLoad': '5', 'FW_VERSION': '1.361', 'overStatus': '0', 'Duration': '1000', 'ICON': 'b-icon-curtain_01', 'delay': 3810}
# things: [{'TIMESTAMP': '1619179952875', 'NAME': 'Blind', 'SERIAL': '00140d6f1950f166', 'MODEL': 'BEAKR1601', 'requestPosition': '100', 'currentPosition': '100', 'moveState': '0', 'setLoad': '2418', 'currentLoad': '5', 'FW_VERSION': '1.361', 'overStatus': '0', 'Duration': '1000', 'ICON': 'b-icon-curtain_01', 'thingUri': '/hub/00140d6f1950f166', 'PERMISSION_TYPE': '"ownership"', 'delay': 2594}]

_LOGGER = logging.getLogger(__name__)

MAPPING = {
    "NAME": "name",
    "thingUri": "thing_uri",
    "MODEL": "model",
    "FW_VERSION": "fw_version",
    "requestPosition": "request_position",
    "TIMESTAMP": "timestamp",
    "SERIAL": "serial",
    "currentPosition": "current_position",
    "moveState": "move_state",
    "setLoad": "set_load",
    "currentLoad": "current_load",
    "overStatus": "over_status",
    "Duration": "duration",
    "ICON": "icon",
    "delay": "delay",
    "PERMISSION_TYPE": "permission_type",
    "resave": "resave",
    "resaveflag": "resave_flag",
    "buttonControl": "button_control",
}


@dataclass
class Thing:
    """Class for representing Things."""

    name: str
    model: str
    fw_version: str
    thing_uri: str | None = None
    request_position: int = 0
    current_position: int = 0
    move_state: int = 0
    timestamp: int | None = None
    serial: str | None = None
    set_load: int | None = None
    current_load: int | None = None
    over_status: int | None = None
    duration: int | None = None
    icon: str | None = None
    delay: int | None = None
    permission_type: str | None = None
    datetime: datetime | None = None
    resave: str | None = None
    resave_flag: str | None = None
    button_control: str | None = None

    @classmethod
    def create_from_dict(cls, input_dict: dict[str, Any]) -> Thing:
        """Create a Thing from a dict."""
        _LOGGER.debug("Creating Thing from dict: %s", input_dict)
        class_fields = {f.name: f.type for f in fields(cls)}
        thing = {}
        for key, value in input_dict.items():
            new_key = MAPPING.get(key)
            if new_key is None:
                _LOGGER.info("%s not in Thing mapping fields", key)
                continue
            if new_key not in class_fields:
                _LOGGER.info("%s not in Thing class fields", key)
                continue
            if class_fields[new_key] in ("int", "int | None"):
                try:
                    value = int(value)
                except ValueError:
                    _LOGGER.warning("%s not an int, value was: %s", key, value)
                    continue
            thing[new_key] = value
        return Thing(**thing)

    def __post_init__(self) -> None:
        """Do post init work."""
        if self.timestamp is not None:
            self.datetime = datetime.utcfromtimestamp(self.timestamp / 1000)
        if self.thing_uri is None and self.serial is not None:
            self.thing_uri = f"/hub/{self.serial}"

    def compare_name(self, name: str) -> bool:
        """Compare name to name."""
        return self.name == name



MAIN_HOST = "https://sky.brunt.co"
MAIN_THINGS_PATH = {"path": "/thing", "host": MAIN_HOST}
THINGS_HOST = "https://thing.brunt.co:8080"
REQUEST_POSITION_KEY = "requestPosition"
DT_FORMAT_STRING = r"%a, %d-%b-%Y %H:%M:%S %Z"
COOKIE_DOMAIN = "brunt.co"


"""Test script."""
import asyncio
import logging
from aiohttp import ClientSession
from aiohttp.client_exceptions import ClientResponseError
import json
from datetime import datetime
import time

from brunt import BruntClientAsync, Thing
from brunt.utils import RequestTypes

logging.basicConfig(level=logging.DEBUG)
_LOGGER = logging.getLogger(__name__)


async def main():
    """Run Main Test method."""
    creds = json.loads(open("tests//creds.json").read())
    # cases = [None, "password", "username", "both", "thingUri"]
    cases = ["thingUri"]
    for case in cases:
        username = creds["username"]
        password = creds["password"]
        bapi = BruntClientAsync(
            username="failname" if case in ["username", "both"] else username,
            password="failpass" if case in ["password", "both"] else password,
        )
        print("Calling Brunt")
        try:
            await bapi.async_login()
            if case == "thingUri":
                things = await bapi.async_get_things()
                _LOGGER.debug("Things: %s", things)
                _LOGGER.debug("ThingUri: %s", things[0].thingUri)
                await bapi._http.async_request(
                    bapi._prepare_state(thingUri="/hub/1234567890"), RequestTypes.GET
                )
        except Exception as exp:
            _LOGGER.error("Caught error: %s", exp)
        finally:
            await bapi.async_close()


if __name__ == "__main__":
    loop = asyncio.get_event_loop()
    loop.run_until_complete(main())



"""Main code for brunt http."""
import json
import logging
from abc import abstractmethod, abstractproperty
from datetime import datetime
from typing import Union

import requests
from aiohttp import ClientSession
from multidict import CIMultiDict
from requests.utils import CaseInsensitiveDict

from .const import COOKIE_DOMAIN, DT_FORMAT_STRING
from .utils import RequestTypes

_LOGGER = logging.getLogger(__name__)

DEFAULT_HEADER = CaseInsensitiveDict(
    {
        "Content-Type": "application/x-www-form-urlencoded; charset=UTF-8",
        "Origin": "https://sky.brunt.co",
        "Accept-Language": "en-gb",
        "Accept": "application/vnd.brunt.v1+json",
        "User-Agent": "Mozilla/5.0 (iPhone; CPU iPhone OS 11_3 like Mac OS X) \
AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E216",
    }
)


class BaseBruntHTTP:
    """Base class for Brunt HTTP."""

    @staticmethod
    def _prepare_request(data: dict) -> dict:
        """Prepare the payload and add the length to the header, payload might be empty."""
        payload = ""
        headers = {}
        if "data" in data:
            payload = json.dumps(data["data"])
            headers = {"Content-Length": str(len(payload))}

        return {"url": data["host"] + data["path"], "data": payload, "headers": headers}

    @abstractmethod
    def request(self, data: dict, request_type: RequestTypes) -> Union[dict, list]:
        """Return the request response - abstract."""

    @abstractmethod
    async def async_request(
        self, data: dict, request_type: RequestTypes
    ) -> Union[dict, list]:
        """Return the request response - abstract."""

    @abstractproperty
    def is_logged_in(self) -> bool:
        """Return True if there is a session and the cookie is still valid."""


class BruntHttp(BaseBruntHTTP):
    """Class for brunt http calls."""

    def __init__(self, session: requests.Session = None):
        """Initialize the BruntHTTP object."""
        self.session = session if session else requests.Session()
        self.session.headers = DEFAULT_HEADER

    @property
    def is_logged_in(self) -> bool:
        """Return True if there is a session and the cookie is still valid."""
        if not self.session.cookies:
            return False

        for cookie in self.session.cookies:
            if cookie.domain == COOKIE_DOMAIN:
                if cookie.expires is not None:
                    return (
                        datetime.strptime(str(cookie.expires), DT_FORMAT_STRING)
                        > datetime.utcnow()
                    )
        return False

    async def async_request(
        self, data: dict, request_type: RequestTypes
    ) -> Union[dict, list]:
        """Raise error for using this call with sync."""
        raise NotImplementedError("You are using the sync version, please use request.")

    def request(self, data: dict, request_type: RequestTypes) -> Union[dict, list]:
        """Request the data.

        :param session: session object from the Requests package
        :param data: internal data of your API call
        :param request: the type of request, based on the RequestType enum
        :returns: dict with sessionid for a login and the dict of the things for the other calls,
            or just success for PUT
        :raises: raises errors from Requests through the raise_for_status function
        """
        resp = self.session.request(
            request_type.value, **BaseBruntHTTP._prepare_request(data)
        )
        # raise an error if it occured in the Request.
        resp.raise_for_status()
        # check if there is something in the response body
        if len(resp.text) == 0:
            return {"result": "success"}
        return resp.json()


class BruntHttpAsync(BaseBruntHTTP):
    """Class for async brunt http calls."""

    def __init__(self, session: ClientSession = None):
        """Initialize the BruntHTTP object."""
        self.session = session if session else ClientSession()
        self.session._default_headers = CIMultiDict(DEFAULT_HEADER)

    @property
    def is_logged_in(self) -> bool:
        """Return True if there is a session and the cookie is still valid."""
        if not self.session.cookie_jar:
            return False
        for cookie in self.session.cookie_jar:
            if cookie.get("domain") == COOKIE_DOMAIN:
                if cookie.get("expires") is not None:
                    return (
                        datetime.strptime(str(cookie.get("expires")), DT_FORMAT_STRING)
                        > datetime.utcnow()
                    )
        return False

    def request(self, data: dict, request_type: RequestTypes) -> Union[dict, list]:
        """Raise error for using this call with async."""
        raise NotImplementedError(
            "You are using the Async version, please use async_request."
        )

    async def async_request(
        self, data: dict, request_type: RequestTypes
    ) -> Union[dict, list]:
        """Request the data.

        :param session: session object from the Requests package
        :param data: internal data of your API call
        :param request: the type of request, based on the RequestType enum
        :returns: dict with sessionid for a login and the dict of the things for
            the other calls, or just success for PUT
        :raises: raises errors from Requests through the raise_for_status function
        """
        async with self.session.request(
            request_type.value,
            **BaseBruntHTTP._prepare_request(data),
            raise_for_status=True
        ) as resp:
            try:
                return await resp.json(content_type=None)
            except json.JSONDecodeError:
                return {"result": "success"}
